pro calcBnu, temp, lambda, result

  ; Calculates the Planck Law in CGS units
  ;
  ; Input:
  ; temp = temperature in Kelvin
  ; lambda = wavelength in microns
  ;
  ; Output:
  ; result = Bnu in erg cm^-2 steradian^-1
  ;
  ; Created by Chris Stark
  ; University of Maryland Physics Dept/Goddard Space Flight Center
  ; starkc@umd.edu
  ; Last updated 08/09/07 by Chris Stark

  const1 = 3.972895E-4 ;const1 = 2 * h * c * (1E6)^3 (in CGS w/ correction for microns)
  const2 = 1.438769E4 ;const2 = h * c * 1E6 / k (in CGS w/ correction for microns)
  result = const1 / (lambda^3.0 * (exp(const2 / (lambda * temp)) - 1))
end



function calc_exozodi_flux, M_V, vmag, F0V, nexozodis, lambda, lambdamag, F0lambda

;Inputs
;M_V  = V band absolute magnitude of stars
;vmag = V band apparent magnitude of stars
;F0V  = V band flux zero point of stars
;Lstar = bolometric luminosity of stars
;lambda = wavelength in microns (vector of length nlambda)
;lambdamag = apparent magnitude of stars at each lambda (array nstars x nlambda)
;F0lambda = flux zero point at wavelength lambda (vector of length nlambda)

;Output
;exozodi surface brightness photons s^-1 cm^-2 arcsec^-2 nm^-1 / F0
;i.e., it returns 10^(-0.4*magOmega_EZ)
; - multiply by F0 to get photons s^-1 cm^-2 arcsec^-2 nm^-1
; - multiply by energy of photons to get erg s^-1 cm^-2 arcsec^-2 nm^-1

if n_elements(lambda) ne n_elements(F0lambda) then stop,'ERROR. F0lambda and lambda must be vectors of identical length.'
x = size(lambdamag,/dim)
if n_elements(lambda) gt 1 then begin
   if x[1] ne n_elements(lambda) then stop,'ERROR. lambdamag must have dimensions nstars x nlambda, where nlambda is the length of lambda.'
endif

vmag_1zodi = 22. ;V band surface brightness of 1 zodi in mag arcsec^-2
vflux_1zodi = 10.^(-0.4*vmag_1zodi) ;V band flux (modulo the F0 factor out front)

M_V_sun = 4.83 ;V band absolute magnitude of Sun

;Calculate counts s^-1 cm^-2 arcsec^-2 nm^-1 @ V band
;The following formula maintains a constant optical depth regardless
;of stellar type
;NOTE: the older version of the code used the following expression
;flux_exozodi_Vband = F0V * (double(10.)^(-0.4*(M_V - M_V_sun)) / Lstar) * (nexozodis * vflux_1zodi)
;The above expression has a 1/Lstar in it, which corrects for the
;1/r^2 factor based on the EEID.  In the new version of exposure_time_calculator.c,
;we explicitly divide by 1/r^2, so the new version of the
;eflux_exozodi_Vband expression does not divide by Lstar.
flux_exozodi_Vband = F0V * double(10.)^(-0.4*(M_V - M_V_sun)) * (nexozodis * vflux_1zodi) ;vector of length nstars

;Now, multiply by the ratio of the star's counts received at lambda to those received at V band
nstars = n_elements(vmag)
nlambda = n_elements(lambda)
flux_exozodi_lambda = congrid([[flux_exozodi_Vband],[flux_exozodi_Vband]],nstars,nlambda)
for ilambda=0,nlambda-1 do begin
   flux_exozodi_lambda[*,ilambda] *= (F0lambda[ilambda] * double(10.)^(-0.4*lambdamag[*,ilambda])) / (F0V * double(10.)^(-0.4*vmag))
endfor

;Now, because we return just the quantity 10.^(-0.4*magOmega_EZ), we remove the F0...
for ilambda=0,nlambda-1 do begin
   flux_exozodi_lambda[*,ilambda] /= F0lambda[ilambda]
endfor


return, flux_exozodi_lambda

end
