function calc_flux_zero_point,lambda,jy=jy,cgs=cgs,pcgs=pcgs,perlambda=perlambda,AB=AB,verbose=verbose

;This code calculates the flux zero point for a given wavelength
  
;By default, it returns Johnson zero points
;By setting the /ab keyword, it will return AB flux zero point

;The Johnson zero points come from Table A.2 from the Spitzer Telescope Handbook at
;http://irsa.ipac.caltech.edu/data/SPITZER/docs/spitzermission/missionoverview/spitzertelescopehandbook/19/
;That table is as follows:
;Passband	Effective wavelength (microns)	Johnson Zero point (Jy)
;U	0.36	1823
;B	0.44	4130
;V	0.55	3781
;R	0.71	2941
;I	0.97	2635
;J	1.25	1603
;H	1.60	1075
;K	2.22	667
;L	3.54	288
;M	4.80	170
;N	10.6	36
;O	21.0	9.4

;The AB zero points are simply calculated as 5.5099e6 / lambda
;3.6308e-20
  
;INPUT
;lambda (microns)
;jy: flag to make the output value in Jy
;cgs: flag to make the output value in erg s^-1 cm^-2 Hz^-1
;pcgs: flag to make the output value in photons s^-1 cm^-2 Hz^-1
;perlambda: flag to convert all output values to cm^-1 instead of Hz^-1

  
;OUTPUT
;f0_jy (flux zero point in Jy)
;f0_photons_cgs (flux zero point in photons s^-1 cm^-2 Hz^-1)


  if n_elements(jy) eq 0 then jy = 0
  if n_elements(cgs) eq 0 then cgs = 0
  if n_elements(pcgs) eq 0 then pcgs = 0
  if n_elements(perlambda) eq 0 then perlambda = 0
  if n_elements(verbose) eq 0 then verbose = 0
  if (jy + cgs + pcgs) eq 0 then stop,'Must specificy output units: /jy, /cgs, /pcgs, /perlambda'
  if (jy + cgs + pcgs) gt 1 then stop,'Must specificy only one unit flag.'
  if jy and perlambda then stop,'Cannot set Jy and perlambda'
  

  ;Calculate the zero point
  if keyword_set(ab) then begin
     ;AB magnitude system
     f0 = 3.6308e3 ;This is the zero point Jy
  endif else begin
     ;Johnson magnitude system
     known_lambda = [0.36,0.44,0.55,0.71,0.97,1.25,1.60,2.22,3.54,4.80,10.6,21.0]
     known_zeropoint_jy = [1823,4130,3781,2941,2635,1603,1075,667,288,170,36,9.4]
     ;Now we interpolate to lambda
     ;Note that the interpolation is best done in log-log space
     logf0 = interpol(alog10(known_zeropoint_jy),alog10(known_lambda),alog10(lambda),/quadratic) ;in jy
     f0 = 10.^logf0 ;undo the logarithm
  endelse

  
  
  ;Now change the units from Jy if necessary
  c = 2.998e10                  ;cm s^-1
  h = 6.62608e-27               ;planck constant in cgs
  lambda_cm = lambda / 1e4
  ephoton_cgs = h * c/ lambda_cm
  if cgs then f0 = double(f0)*1e-23 ;convert to CGS
  if pcgs then f0 = double(f0)*1e-23 / ephoton_cgs ;convert to # photons in CGS
  if perlambda then f0 *= (c / lambda_cm^2.)
  

  if jy then unittext = 'Jy'
  if cgs and not perlambda then unittext = 'erg s^-1 cm^-2 Hz^-1 (CGS per unit frequency)'
  if pcgs and not perlambda then unittext = 'photons s^-1 cm^-2 Hz^-1 (CGS per unit frequency)'
  if cgs and perlambda then unittext = 'erg s^-1 cm^-3 (CGS per unit wavelength)'
  if pcgs and perlambda then unittext = 'photons s^-1 cm^-3 (CGS per unit wavelength)'

  if verbose then print,'Johnson flux zero point calculated at '+strcompress(string(lambda,format='(F0.2)'))+' microns in units of '+unittext

  
  return, f0
end
