pro calcblambda, temp, lambda, result

  ; Calculates the Planck Law (per unit wavelength) in CGS units
  ;
  ; Input:
  ; temp = temperature in Kelvin
  ; lambda = wavelength in microns
  ;
  ; Output:
  ; result = Bnu in erg cm^-3 s^-1 steradian^-1
  ;
  ; Created by Chris Stark
  ; Goddard Space Flight Center
  ; christopher.c.stark@nasa.gov
  ; Last updated 01/28/15 by Chris Stark

  const1 = 1.1910439E15 ;const1 = 2 * h * c^2 * (1E4)^5 (in CGS w/ correction for microns)
  const2 = 1.438769E4 ;const2 = h * c * 1E4 / k (in CGS w/ correction for microns)
  result = const1 / (lambda^5.0 * (exp(const2 / (lambda * temp)) - 1))
end



function calc_zodi_flux, dec, ra, lambda, F0, starshade=starshade, ss_elongation=ss_elongation

;Inputs
;Dec = declination of target in degrees (J2000 equatorial coordinate)
;RA = right ascension of target in degrees (J2000 equatorial coordinate)
;lambda = wavelength in microns (vector of length nlambda)
;starshade = setting this flag turns on starshade mode
;ss_elongation = mean solar elongation for the starshade's observations (degrees)
  
;Output
;zodi surface brightness photons s^-1 cm^-2 arcsec^-2 nm^-1 / F0
;i.e., it returns 10^(-0.4*magOmega_ZL)
; - multiply by F0 to get photons s^-1 cm^-2 arcsec^-2 nm^-1
; - multiply by energy of photons to get erg s^-1 cm^-2 arcsec^-2 nm^-1
;output has dimensions nstars x nlambda

if n_elements(lambda) ne n_elements(F0) then stop,'ERROR. F0 and lambda must be vectors of identical length.'
if keyword_set(starshade) and n_elements(ss_elongation) eq 0 then stop,'ERROR. You have set the STARSHADE flag.  Must specify SS_ELONGATION in degrees.'
if not keyword_set(starshade) and n_elements(ss_elongation) ne 0 then stop,'ERROR. You must enable STARSHADE mode if you are setting SS_ELONGATION in degrees.'
  
;Convert to radians internally
dec_rad = dec*(!dpi/180.)
ra_rad = ra*(!dpi/180.)

;First, convert equatorial coordinates to ecliptic coordinates
;This is nicely summarized in Leinert et al. (1998)
eps = 23.439 ;J2000 obliquity of the ecliptic in degrees
eps_rad = eps*(!dpi/180.) ;convert to radians
sinbeta = sin(dec_rad) * cos(eps_rad) - cos(dec_rad) * sin(eps_rad) * sin(ra_rad) ;all we need is the sine of the latitude
sinbeta0 = sinbeta              ;contains the +/- values of beta
sinbeta = abs(sinbeta)          ;f135 below is symmetric about beta=0
sinbeta2 = sinbeta*sinbeta
sinbeta3 = sinbeta2*sinbeta

;print,'beta = ',asin(sinbeta)*180./!pi

;Here are the solar longitude and beta values for Table 17 from
;Leinert et al. (1998)
beta_vector = [0.,5,10,15,20,25,30,45,60,75]
beta_array = [[beta_vector],[beta_vector]]
beta_array = rebin(beta_array,10,20)
beta_array = beta_array[*,0:18]
sollong_vector = [0,5,10,15,20,25,30,35,40,45,60,75,90,105,120,135,150,165,180.]
sollong_array = [[sollong_vector],[sollong_vector]]
sollong_array = rebin(sollong_array,19,10)
sollong_array = rotate(sollong_array,1)

;Here are the values in Table 17
table17 = sollong_array
table17[*,0] = [-1,	-1,	-1,	3140,	1610,	985,	640,	275,	150,	100]
table17[*,1] = [-1,	-1,	-1,	2940,	1540,	945,	625,	271,	150,	100]
table17[*,2] = [-1,	-1,	4740,	2470,	1370,	865,	590,	264,	148,	100]
table17[*,3] = [11500,	6780,	3440,	1860,	1110,	755,	525,	251,	146,	100]
table17[*,4] = [6400,	4480,	2410,	1410,	910,	635,	454,	237,	141,	99]
table17[*,5] = [3840,	2830,	1730,	1100,	749,	545,	410,	223,	136,	97]
table17[*,6] = [2480,	1870,	1220,	845,	615,	467,	365,	207,	131,	95]
table17[*,7] = [1650,	1270,	910,	680,	510,	397,	320,	193,	125,	93]
table17[*,8] = [1180,	940,	700,	530,	416,	338,	282,	179,	120,	92]
table17[*,9] = [910,	730,	555,	442,	356,	292,	250,	166,	116,	90]
table17[*,10] = [505,	442,	352,	292,	243,	209,	183,	134,	104,	86]
table17[*,11] = [338,	317,	269,	227,	196,	172,	151,	116,	93,	82]
table17[*,12] = [259,	251,	225,	193,	166,	147,	132,	104,	86,	79]
table17[*,13] = [212,	210,	197,	170,	150,	133,	119,	96,	82,	77]
table17[*,14] = [188,	186,	177,	154,	138,	125,	113,	90,	77,	74]
table17[*,15] = [179,	178,	166,	147,	134,	122,	110,	90,	77,	73]
table17[*,16] = [179,	178,	165,	148,	137,	127,	116,	96,	79,	72]
table17[*,17] = [196,	192,	179,	165,	151,	141,	131,	104,	82,	72]
table17[*,18] = [230,	212,	195,	178,	163,	148,	134,	105,	83,	72]



;For a coronagraph, we assume star can be observed near sollong ~ 135 degrees
;The old f135 calculation.  Now we interpolate the above table.
;I135_o_I90 = 0.69
;f135 = 1.02331 - 0.565652*sinbeta - 0.883996*sinbeta2 + 0.852900*sinbeta3
;f = I135_o_I90 * f135
j = where(abs(sollong_vector-135.) eq min(abs(sollong_vector-135.))) & j = j[0]
k = where(abs(sollong_vector-90.) eq min(abs(sollong_vector-90.))) & k = k[0]
f = interpol(table17[*,j]/table17[0,k],sin(beta_vector*!pi/180.),sinbeta)


;For a starshade, we assume a mean solar elongation of 59 degrees
if keyword_set(starshade) then begin

   
   ;First, calculate inclination to point at beta of all targets
   inclination = asin(sinbeta0 / sin(ss_elongation*!pi/180.))*180./!pi
   cossollong = (cos(ss_elongation*!pi/180.)/cos(asin(sinbeta))) ;solar longitude of targets at elongation of ss_elongation
   sinsollong = cos(inclination*!pi/180.) * sin(ss_elongation*!pi/180.) / cos(asin(sinbeta0))
   sollong = atan(sinsollong,cossollong)*180/!pi
   ;Now that we have the solar longitude and beta of every target at a
   ;solar elongation of ss_elongation, we can calculate their zodi values
   sollong_indices = sollong    ;first, just make some arrays
   beta_indices = sollong
   j = where(sollong lt 45)
   if j[0] ne -1 then sollong_indices[j] = sollong[j]/5.
   j = where(sollong ge 45)
   if j[0] ne -1 then sollong_indices[j] = 9.+(sollong[j]-45.)/15.
   beta = abs(asin(sinbeta)*180./!pi)
   j = where(beta lt 30)
   if j[0] ne -1 then beta_indices[j] = beta[j]/5.
   j = where(beta ge 30)
   if j[0] ne -1 then beta_indices[j] = 6.+(beta[j]-30.)/15.
   f = interpolate(table17/table17[0,k],beta_indices,sollong_indices)
endif

;wavelength dependence
;The following comes from fits to Table 19 in Leinert et al 1998
zodi_lambda = [0.2,0.3,0.4,0.5,0.7,0.9,1.0,1.2,2.2,3.5,4.8,12,25,60,100,140]  ;microns
zodi_blambda = [2.5e-8,5.3e-7,2.2e-6,2.6e-6,2.0e-6,1.3e-6,1.2e-6,8.1e-7,1.7e-7,5.2e-8,1.2e-7,7.5e-7,3.2e-7,1.8e-8,3.2e-9,6.9e-10] ;W m^-2 sr^-1 micron^-1
;convert the above to erg s^-1 cm^-2 arcsec^-2 nm^-1
zodi_blambda *= 1e7 ;convert W to erg s^-1
zodi_blambda /= 1e4 ;convert m^-2 to cm^-2
zodi_blambda /= 4.25e10 ;convert sr^-1 to arcsec^-2
zodi_blambda /= 1000.   ;convert micron^-1 to nm^-1
;It's better to interpolate w/ a quadratic in log-log space
blambda = interpol(alog10(zodi_blambda),alog10(zodi_lambda),alog10(lambda),/quadratic)
blambda = 10.^blambda
I90fabsfco = blambda

;Now divide by energy of a photon in erg
c = 2.998e10 ;cm s^-1
h = 6.62608e-27 ;planck constant in cgs
ephoton = h * c/(lambda/1e4)
I90fabsfco /= ephoton
I90fabsfco /= F0

;Multiply by the wavelength dependence for each value of lambda
nstars = n_elements(ra)
nlambda = n_elements(lambda)
flux_zodi = congrid([[f],[f]],nstars,nlambda)
for ilambda=0,nlambda-1 do begin
   flux_zodi[*,ilambda] *= I90fabsfco[ilambda]
endfor

mag_zodi = -2.5*alog10(flux_zodi)


return, flux_zodi

end
