pro calculate_exposure_time, t_exp

;Calculates the exposure time, t_exp (seconds), of a planet
;observed with a "toy" coronagraph defined by IWA, OWA,
;contrast, noisefloor, bandwidth, Tcore, and TLyot.

;This calls "expsoure_time_calculator.c," which is the C routine
;used by AYO. This code has been benchmarked using the ESYWG/CDS/CTR
;Exposure Time Comparison spreadsheet.

;Created by Christopher Stark
;NASA GSFC
;Last updated 17 June 2024
  
;-------- INPUTS ---------
;Target star parameters
Lstar = [1.83] ;luminosity of star (solar luminosities) ;ntargs array
dist = [18.2]  ;distance to star (pc) ;ntargs array
vmag = [5.444] ;stellar mag at V band ;ntargs array
mag = [5.687]                     ;stellar mag at desired lambda ;ntargs x nlambda array
angdiam_arcsec = [0.01]          ;angular diameter of star (arcsec) ;ntargs array
nzodis = [3.0] ;amount of exozodi around target star ("zodis") ;ntargs array
ra = [101.5589542028] ;right ascension of target star used to estimate zodi (deg) ;ntargs array
dec = [79.56481016336] ;declination of target star used to estimate zodi (deg) ;ntargs array

;Planet parameters
sp = [0.0742] ;separation of planet (arcseconds) ;nmeananom x norbits x ntargs array
deltamag = [25.5] ;difference in mag between planet and host star ;nmeananom x norbits x ntargs array
min_deltamag = [25.0] ;brightest planet to resolve w/ photon counting detector evaluated at the IWA, sets the time between counts ;ntargs array

;Observational parameters
lambda = [0.5] ;wavelength ;nlambda array
SR = [5.0] ;spec res ;nlambda array
SNR = [7.0] ;signal to noise ;nlambda array
throughput = [0.36] ;throughput not incl. coronagraph core throughput ;nlambda array
photap_rad = 0.822 ;(lambda/D) ;scalar

;Telescope & spacecraft parameters
D = 7.87 ;circumscribed diameter of aperture (m) ;scalar
Area = !pi/4.*D^2.*(1.0-0.121) ;effective collecting area of telescope (m^2) ;scalar
toverhead_fixed = 8.25e3 ;fixed overhead time (seconds)
toverhead_multi = 1.1         ;multiplicative overhead time

;Instrument parameters
IWA = 3.0   ;smallest WA to allow (lambda/D) ;scalar
OWA = 100.0 ;largest WA to allow (lambda/D) ;scalar
contrast = 1.05e-13 ;contrast of coronagraph (uniform over dark hole and unitless) ;scalar
noisefloor = 0.03*contrast ;1 sigma systematic noise floor expressed as a contrast (uniform over dark hole and unitless) ;scalar
bandwidth = 0.2 ;fractional bandwidth of coronagraph (unitless)
Tcore = 0.4  ;core throughput of coronagraph (uniform over dark hole, unitless) ;scalar
TLyot = 0.65;1.6*Tcore ;Lyot transmission of the coronagraph and the factor of 1.6 is just an estimate, used for skytrans

;Detector parameters
det_pixscale_mas = 0.5*(0.5e-6/D) * (180./!dpi*60.*60.*1000.) ;detector pixel scale (mas) ;scalar
det_npix_multiplier = [1] ;# of detector pixels per image plane "pixel";nlambda array, 1 for detections or spectra w/ ERD, ~6*(140/SR) for spectra with IFS
det_DC = [3e-5] ;dark current (counts pix^-1 s^-1) ;nlambda array
det_RN = [0.0] ;read noise (counts pix^-1 read^-1) ;nlambda array
det_tread = [1000.] ;read time (s) ;nlambda array
det_CIC = [1.3e-3] ;clock induced charge (counts pix^-1 photon_count^-1) ;nlambda array
;-------- END OF INPUTS ---------





;Misc parameters that probably don't need to be changed
psf_trunc_ratio = [0.3] ;array
td_limit = 1e20 ;limit placed on exposure times (something really large for this code) ;scalar
nooptimize = 0 ;do not attempt to optimize exposure times for this code ;scalar
optimize_phase = 0 ;optimize the phase of the planet (does not work in this code) ;scalar

;"Create" coronagraph1, the only coronagraph used in this code
enable_coro1 = 1
pixscale1 = 0.25 ;lambda/D
npix1 = 2*60/pixscale1
xcenter1 = npix1/2.0
ycenter1 = npix1/2.0
bw1 = bandwidth
angdiams1 = [0.0,10.0]
ndiams1 = 2
npsfratios1 = 1
nrolls1 = 1
r = rgen(npix1,npix1)*pixscale1 ;create an array of circumstellar separations in units of lambda/D centered on star
j = where(r lt IWA or r gt OWA)                        ;find separations interior to IWA or exterior to OWA
omega_lod1 = fltarr(npix1,npix1) + !pi * photap_rad^2. ;size of photometric aperture at all separations
skytrans1 = fltarr(npix1,npix1)+TLyot ;skytrans at all separations
photap_frac1 = fltarr(npix1,npix1)+Tcore ;core throughput at all separations
if j[0] ne -1 then photap_frac1[j] = 0.0
PSFpeak = 0.025 * TLyot ;this is an approximation based on PAPLC results
Istar1 = fltarr(npix1,npix1) + contrast*PSFpeak
noisefloor1 = fltarr(npix1,npix1) + noisefloor*PSFpeak
Istar1 = [[[Istar1]],[[Istar1]]]
noisefloor1 = [[[noisefloor1]],[[noisefloor1]]]


;We only use one coronagraph in this code, so initialize the unused variables
initialize_unused_coronagraph, enable_coro2, Istar2, noisefloor2, photap_frac2, omega_lod2, skytrans2, pixscale2, npix2, xcenter2, ycenter2, bw2, angdiams2, ndiams2, npsfratios2, nrolls2
initialize_unused_coronagraph, enable_coro3, Istar3, noisefloor3, photap_frac3, omega_lod3, skytrans3, pixscale3, npix3, xcenter3, ycenter3, bw3, angdiams3, ndiams3, npsfratios3, nrolls3
initialize_unused_coronagraph, enable_coro4, Istar4, noisefloor4, photap_frac4, omega_lod4, skytrans4, pixscale4, npix4, xcenter4, ycenter4, bw4, angdiams4, ndiams4, npsfratios4, nrolls4

;Some things specific to this code
ntargs = 1
nlambda = 1
ntot = 1
nmeananom = 1
norbits = 1
xp = sp
yp = sp*0.0

;Initialize some arrays needed for outputs...
besticoro = [0] 
bestilambda = [0]
avgpsfomega = [0.0]
minpsfomega = [0.0]
maxpsfomega = [0.0]
avgpsftruncratio = [0.0]
minpsftruncratio = [0.0]
maxpsftruncratio = [0.0]
tp = [0.0] ;exposure time of every planet

;Calculate zodi and exozodi fluxes
F0V = (calc_flux_zero_point(0.55,/pcgs,/perlambda) / 1e7)       ;convert to photons cm^-2 nm^-1 s^-1
F0 = (calc_flux_zero_point(lambda,/pcgs,/perlambda) / 1e7)      ;convert to photons cm^-2 nm^-1 s^-1
M_V = vmag - 5.0 * alog10(dist) + 5.0 ;calculate absolute V band mag of target
Fzodi_list = calc_zodi_flux(dec, ra, lambda, F0)
Fexozodi_list = calc_exozodi_flux(M_V, vmag, F0V, nzodis, lambda, mag, F0)
Fbinary_list = [0.0] ;this code ignores stray light from binaries
Fp0 = 10.^(-0.4*deltamag)       ;flux of planet



;Make sure the variables are the right data type for C code
format_for_exposure_time_calculator, ntargs, nlambda, $
                                     Lstar, dist, angdiam_arcsec, min_deltamag,  $
                                     mag, Fzodi_list, Fexozodi_list, Fbinary_list, $
                                     ntot, nmeananom, norbits, Fp0, sp, xp, yp, $
                                     lambda, SR, SNR, F0, throughput, $
                                     det_pixscale_mas, det_npix_multiplier, det_DC, det_RN, det_tread, det_CIC, $
                                     D, Area, IWA, OWA, photap_rad, psf_trunc_ratio, $
                                     enable_coro1, Istar1, noisefloor1, photap_frac1, omega_lod1, skytrans1, pixscale1, npix1, xcenter1, ycenter1, bw1, angdiams1, ndiams1, npsfratios1, nrolls1, $
                                     enable_coro2, Istar2, noisefloor2, photap_frac2, omega_lod2, skytrans2, pixscale2, npix2, xcenter2, ycenter2, bw2, angdiams2, ndiams2, npsfratios2, nrolls2,  $
                                     enable_coro3, Istar3, noisefloor3, photap_frac3, omega_lod3, skytrans3, pixscale3, npix3, xcenter3, ycenter3, bw3, angdiams3, ndiams3, npsfratios3, nrolls3,  $
                                     enable_coro4, Istar4, noisefloor4, photap_frac4, omega_lod4, skytrans4, pixscale4, npix4, xcenter4, ycenter4, bw4, angdiams4, ndiams4, npsfratios4, nrolls4,  $
                                     toverhead_fixed, toverhead_multi, td_limit, optimize_phase, nooptimize, $
                                     besticoro, bestilambda, avgpsfomega, minpsfomega, maxpsfomega, avgpsftruncratio, minpsftruncratio, maxpsftruncratio, tp

                                            
;Now calculate the exposure times
routinepath=routine_info('calculate_exposure_time',/source)  ;Determine where this code is located
routinepath=strmid(routinepath.path,0,strlen(routinepath.path)-strlen('calculate_exposure_time')-4) ;modify the string a bit to just get the full path directory
returnval = CALL_EXTERNAL(routinepath+'exposure_time_calculator.so', 'exposure_time_calculator', $
                          ntargs, nlambda, $
                          Lstar, dist, angdiam_arcsec, min_deltamag, $
                          mag, Fzodi_list, Fexozodi_list, Fbinary_list, $
                          ntot, nmeananom, norbits, Fp0, sp, xp, yp, $
                          lambda, SR, SNR, F0, throughput, $
                          det_pixscale_mas, det_npix_multiplier, det_DC, det_RN, det_tread, det_CIC, $
                          D, Area, IWA, OWA, photap_rad, psf_trunc_ratio, $
                          enable_coro1, Istar1, noisefloor1, photap_frac1, omega_lod1, skytrans1, pixscale1, npix1, xcenter1, ycenter1, bw1, angdiams1, ndiams1, npsfratios1, nrolls1, $
                          enable_coro2, Istar2, noisefloor2, photap_frac2, omega_lod2, skytrans2, pixscale2, npix2, xcenter2, ycenter2, bw2, angdiams2, ndiams2, npsfratios2, nrolls2, $
                          enable_coro3, Istar3, noisefloor3, photap_frac3, omega_lod3, skytrans3, pixscale3, npix3, xcenter3, ycenter3, bw3, angdiams3, ndiams3, npsfratios3, nrolls3, $
                          enable_coro4, Istar4, noisefloor4, photap_frac4, omega_lod4, skytrans4, pixscale4, npix4, xcenter4, ycenter4, bw4, angdiams4, ndiams4, npsfratios4, nrolls4, $
                          toverhead_fixed, toverhead_multi, td_limit, optimize_phase, nooptimize,  $
                          besticoro, bestilambda, avgpsfomega, minpsfomega, maxpsfomega, avgpsftruncratio, minpsftruncratio, maxpsftruncratio, tp)

t_exp = tp[0]

print,t_exp

end
